package searchDesign;

import java.io.*;

/**
 *
 * <p>Beschreibung: Der Parser wird ber die Methode initialize mit einer
 * Stopwort-Datei und einem Crawler initialisiert. Er bearbeitet
 * sequentiell alle Dateien, die der Crawler ihm zur Verfgung stellt
 * mit seiner Methode nextWord. Die Methode nextWord gibt erst null zurck,
 * wenn das letzte Wort der letzten Datei abgearbeitet wurde. Nachdem er alle
 * Dateien abgearbeitet hat, erzeugt der Parser eine docMap-Datei
 * (Methode writeDocMap, s.u.) und schreibt die Pfade der bearbeiteten Dateien,
 * jeweils gefolgt (in einer neuen Zeile) von der Anzahl der eingelesenen
 * Nichtstopwrter.</p>
 * @version 1.0
 */

public interface ParserInterface {

  /**
   * Initialisierungsmethode
   * @param stopWordFileName Der Pfad zu der Datei, die die Stopwrter enthlt
   * @param crawler Der zuvor erzeugte Crawler
   *
   * @throws java.io.FileNotFoundException
   * @throws java.io.IOException
   */
    public void initialize (String stopWordFileName, CrawlerInterface crawler)
                        throws java.io.FileNotFoundException, java.io.IOException;


    /**
     * schreibt in der DocMap-Datei mit Namen fileName folgende Informationen: <br>
     * - in der ersten Zeile: die Anzahl der bearbeiteten Dateien <br>
     * - danach jeweils den Pfad zu einer bearbeiteten Datei, gefolgt in einer
     * eigenen Zeile von der Anzahl der in der betroffenen Datei eingelesenen
     * Nichtstopwrter.<br>
     * Die DocMap-Datei enthlt keine Leerzeilen
     * @param fileName Der Pfad zur DocMap-Datei
     */
    public void writeDocMap(String fileName);

    /**
     * liefert jeweils ein Wort zurck, in Form eines KeyData.
     * Der Parser bercksichtigt nur Nichtstopwrter, d.h. Stopwrter
     * werden berlesen. Bei Nichtstopwrtern gibt der Parser ein KeyData zurck.
     * Ein KeyData enthlt: <br>
     * - den eingelesenen String als key (klein geschrieben), <br>
     * - die Anfangsposition des Wortes, <br>
     * - den Abstand zwischen dem vorigen und dem eingelesen Wort im Text, d.h. die Anzahl
     *   der Nichtwortzeichen zwischen den beiden Wrtern, <br>
     * - die ID (int) des Dokuments, in dem das Wort enthalten ist. <br>
     * Es wird immer der Abstand zum tatschlichen vorigen Wort im Text gespeichert,
     * unabhngig davon, ob dieses ein Stopwort ist oder nicht.
     *
     * @return ein Keydata-Objekt mit oben beschriebenem Inhalt oder null
     * @throws java.io.IOException
     * @throws java.io.FileNotFoundException
     */
    public Data nextWord() throws java.io.IOException, java.io.FileNotFoundException;
}