package searchDesign;

/**
 *
 * <p>Beschreibung: Die QueryEngine bearbeitet die Suchanfragen. </p>
 * @version 1.0
 */

public interface QueryEngineInterface {

  /**
   * Initialisierungsmethode:
   * Liest die Indexdatei indexFile ein und rekonstruiert daraus den
   * invertierten Index. Um Rankingberechnungen und Kontextausgaben machen zu
   * knnen, bentigt die QueryEngine die Informationen aus der Datei
   * docMapFile, die beim Indizieren vom Parser angelegt wurde. In der Datei
   * resultFile werden die Suchergebnisse gespeichert.
   *
   * @param indexFile  Pfad zur Indexdatei
   * @param docMapFile Pfad zur DocMap-Datei
   * @param resultFile Pfad zur Ausgabedatei
   */
    public void initialize (String indexFile, String docMapFile, String resultFile);

    /**
     * Bearbeitet eine Suchanfrage.
     * Term ist die Benutzereingabe, welche eine beliebige Kombination von
     * Wrtern und/oder Wortketten enthlt. Eine Wortkette bezeichnet
     * aufeinander folgende Wrter in doppelten Anfhrungszeichen,
     * z.B. "bse Hexe". term wird in seine einzelnen Bestandteile geparst.
     * Es mssen alle Dokumente ermittelt werden, die alle Wrter des Terms
     * enthalten. Hier sollte man sich alle Dokumente merken, in denen der
     * erste Term enthalten ist, und die Schnittmenge von dieser
     * Dokumentenmenge und den Dokumentenmengen der weiteren Terme der
     * Suchanfrage bilden. Bei der Behandlung von Wortketten muss verifiziert
     * werden, dass die einzelnen Wrter einer Wortkette adjazent sind. Die
     * ntigen Informationen dafr sind in den Fundstellen enthalten. Fr die
     * Ausgabe werden die ermittelten Dokumente nach dem Ranking mit Hilfe von
     * Quicksort sortiert. Anschlieend gibt processQuery die Suchergebnisse
     * sowohl auf dem Bildschirm als auch in die Ausgabedatei resultFile aus. 
     * Fr die Ausgabe der KeyWords im Kontext bietet es sich an, 
     * diesen mittels eines RandomAccessFiles aus den entsprechenden Dokumenten 
     * zu holen. Fr die Ausgabe wird der Originaltext wie in der globalen 
     * Beschreibung unter 3.3.3 beschrieben leicht aufbereitet.
     *
     * @param term Die eingegebene Suchanfrage
     */
    public void processQuery(String term);


}