package searchDesign;

//////////////////////////////////////////////////////////////
/**<p>
* eine moegliche Datenstruktur als Alternative zum eigenen 
* unausgewogenen Binaerbaum oder der Hashtable: eine 
* Erweiterung von java.util.TreeMap (ausgewogener Binaerbaum,
* wird allerdings nicht genutzt, da zur Suche in jedem Fall
* der Baum auf ein Array geschrieben wird und in diesem gesucht
* wird, da dies beim unsortierten Baum ineffizient ist (sein
* kann) und bei der Hashtable notwendig ist.
* Trotz der gewaehrleistung der Ausgewogenheit sehr schnell
* auch beim Einfuegen, bei einigen Test die schnellste der 3 
* alternativen Datenstrukturen
* . Erforderliche Aenderungen lediglich
* die durch das StorageInterface geforderte put(Data) und
* die asDataArray()
* </p>
* @author Fabian Steeg
*/
///////////////////////////////////////////////////////////////

public class StorageTreeMap extends java.util.TreeMap implements StorageInterface{
    
    ////////////////////////////////////////////////////////////////////
    /**
    * Erzeugt aus der Datenstruktur ein Array mit Data-Objekten und gibt
    * dieses zurck.
    *
    * @return Data-Array
    */
    ////////////////////////////////////////////////////////////////////
    
	public Data[] asDataArray(){
	    Object[] a = super.values().toArray();
	    Data[]d = new Data[a.length];
	    for(int i = 0;i<d.length;i++){
	        d[i] = (Data) a[i];
	    }
	    return d;
	}
    
    //////////////////////////////////////////////////////////////////
	/**
    * Fgt newData in die Direktzugriffsstruktur ein. Fall der Key von
    * newData bereits darin enthalten ist, wird die Methode add von
    * Data aufgerufen.
    *
    * @param newData Das neu einzufgende Data-Objekt
    */
    //////////////////////////////////////////////////////////////////
    
	public void put(Data newData){
	    
	    Data schonDa; 
	    
	    if (  (  schonDa = ((Data) super.get(newData.getKey()))  )  != null){
	        schonDa.add(newData);
	        newData = schonDa;
	    }
	    
	    super.put(newData.getKey(), newData);
	}
    
    ////////////////////////////////////////////////////////////////
    /**
    * Gibt den Wert von key zurck, oder null, wenn key nicht in der
    * Direktzugriffsstruktur enthalten ist
    *
    * @param key Der Schlssel, dessen Wert gesucht wird
    * @return Der Wert des gefundenen Schlssels
    */
    /////////////////////////////////////////////////////////////////
    
	public Object get(Object key){
	    return super.get(key);
	}
    
    ///////////////////////////////////////////////
	/**
    * gibt die Anzahl der gespeicherten Keys zurck
    * @return die Anzahl der gespeicherten Keys
    */
    ///////////////////////////////////////////////
    
	public int size(){
	    return super.size();
	}
}